var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const Db = require('./Db');
const util = require('./util');
const autoBind = require('auto-bind');
const _ = require('lodash');
const uuid = require('uuid/v4');

class Dynamo extends Db {
  constructor(props) {
    super(props);

    this.tableName = null;
    this.key = null;

    this.attribute = null;
    this.opts = {};
    this.opts.timestamps = true;
    this.fn = [];

    autoBind(this);
  }

  table(table) {
    this.fn.push(() => this.tableName = table);
    return this;
  }

  item(key) {
    this.fn.push(() => this.key = key || this.key);
    return this;
  }

  ref(path) {
    var fn = () => {
      this.tableName = path.indexOf('/') > -1 ? path.split('/')[0] : path;
      this.key = path.indexOf('/') > -1 ? path.split('/')[1] : this.key;
      return {};
    };
    this.fn.push(fn);
    return this;
  }

  apply() {
    var _this = this;

    return _asyncToGenerator(function* () {
      var { name } = _.last(_this.fn);
      for (let fn of _this.fn) {
        yield fn();
      }
      return _this;
    })();
  }

  get(key) {
    var fn = () => {
      key = key || this.key;
      if (Array.isArray(this.key)) {
        return this.promise = Promise.all(this.key.map(key => this.getItem({
          TableName: this.tableName, Key: { id: key }
        })));
      } else {
        return this.promise = this.getItem({
          TableName: this.tableName, Key: { id: this.key }
        });
      }
    };

    this.fn.push(fn);
    return this;
  }

  list() {
    var fn = () => this.promise = this.scan({
      TableName: this.tableName
    });
    this.fn.push(fn);
    return this;
  }

  select(attr, table) {
    var _this2 = this;

    var fn = (() => {
      var _ref = _asyncToGenerator(function* () {
        var promise = yield _this2.promise;
        _this2.key = _.get(promise, attr);
        _this2.tableName = table || _this2.tableName;
        return {};
      });

      return function fn() {
        return _ref.apply(this, arguments);
      };
    })();
    this.fn.push(fn);
    this.get();
    return this;
  }

  delete(key) {
    var fn = () => {
      key = key || this.key;
      return this.promise = this.deleteItem({
        TableName: this.tableName,
        Key: { id: key }
      }, { id: key });
    };
    this.fn.push(fn);
    return this;
  }

  destroy(keys) {
    var fn = () => {
      return this.promise = Promise.all(keys.map(key => this.deleteItem({
        TableName: this.tableName,
        Key: { id: key }
      }, { id: key })));
    };
    this.fn.push(fn);
    return this;
  }

  update(_ref2) {
    let { id } = _ref2,
        rest = _objectWithoutProperties(_ref2, ['id']);

    if (this.opts.timestamps) {
      rest = _extends({}, rest, { updatedAt: new Date().toString() });
    }
    var fn = () => {
      id = id || this.key;

      const params = _extends({
        TableName: this.tableName,
        Key: { id }
      }, util.mapAttributes(rest), {
        UpdateExpression: `SET ${util.createExpression(rest)}`,
        ReturnValues: 'ALL_NEW'
      });
      return this.promise = this.updateItem(params, _extends({ id }, rest));
    };
    this.fn.push(fn);
    return this;
  }

  filter(args) {
    const params = _extends({}, util.mapAttributes(args), util.createFilterExpression(args));
    var fn = () => {
      params.TableName = this.tableName;
      return this.promise = this.scan(params);
    };

    this.fn.push(fn);
    return this;
  }

  set(args) {
    const Item = _extends({}, args);
    if (this.opts.timestamps) {
      Item.createdAt = new Date().toString();
      Item.updatedAt = new Date().toString();
    }
    var fn = () => {
      Item.id = [this.key, Item.id].reduce((acc, val) => {
        if (val) acc = val;
        return acc;
      }, uuid());
      return this.promise = this.createItem({
        TableName: this.tableName,
        Item
      });
    };
    this.fn.push(fn);
    return this;
  }

}

module.exports = Dynamo;